import collections
import logging
import os
import socket
import threading
import time
import urllib2
from noblogsmv import state
from flask import Flask, redirect, render_template, request, g


app = Flask(__name__)


@app.before_request
def before_request_handler():
    g.sm = app.config.sm


@app.route('/')
def index():
    now = time.time()
    cur_state = g.sm.get_state()
    worker_info = g.sm.get_worker_info()
    state_count = g.sm.state_count
    return render_template('index.html',
                           state=cur_state,
                           state_count=state_count,
                           worker_info=worker_info,
                           now=time.time())


@app.route('/quitquitquit')
def shutdown():
    # This will stop the default Werkzeug development server.
    halt_fn = request.environ.get('werkzeug.server.shutdown')
    if halt_fn:
        halt_fn()
    return 'ok'


def create_app(sm, port, config={}):
    app.config.update(
        SECRET_KEY=os.urandom(8))
    app.config.from_pyfile('APP_CONFIG', silent=True)
    app.config.update(
        HOST_NAME=socket.gethostname(),
        HOST_PORT=port)
    app.config.update(config)
    app.config.sm = sm
    return app


class WsgiThread(threading.Thread):

    def __init__(self, sm, port):
        threading.Thread.__init__(self)
        self.sm = sm
        self.port = port
        self.setDaemon(True)

    def run(self):
        wsgiapp = create_app(self.sm, self.port)
        wsgiapp.run(host='0.0.0.0', port=self.port)

    def stop(self):
        urllib2.urlopen('http://127.0.0.1:%d/quitquitquit' % self.port).read()

